/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of throttle
 *
 * throttle is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * throttle is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stdlib.h>
#include <stdio.h>

#include <bpipe/bpipe.h>
#include <bpipe/bpipe_axaf.h>

#include <randlib_c/randlib.h>

#include <tracefct/tracefct.h>
#include <tracefct/exiterrvals.h>

#include "main.h"
#include "process.h"

void
process( Common c)
{
  unsigned long n;
  BPipe *bpipe;
  BPipeOutput *bpo;
  void *data;

  tf_enter( "process" );

  /*
    if poisson noise requested, generated new number of photons sampled
    from poisson distribution with given number as mean
  */
  if ( c.poisson )
  {
    /* initialize random number generator and get to correct block */
    setall(c.seed1, c.seed2);
    for ( n = 0 ; n < c.block ; n++ )
      initgn(1);

    c.n = ignpoi( c.n );
  }

  bpipe_errno = BPNOERROR;

  if ( NULL == ( bpipe = bpipe_new( ) ) )
    tf_exit( ExitERR_misc, "error creating bpipe: %s",
	     bpipe_strerror( bpipe_errno ) );

  if ( bpipe_input( bpipe, c.input ) )
    tf_exit( ExitERR_misc, "error opening %s for input: %s",
	     c.input, bpipe_strerror( bpipe_errno ) );

  if ( NULL == ( bpo = bpipe_output( bpipe, c.output ) ) )
    tf_exit( ExitERR_misc, "error opening %s for output: %s",
	     c.output, bpipe_strerror( bpipe_errno ) );

  if ( NULL == ( data = ( bpipe_map_alloc( bpipe, 1, NULL ) ) ) )
    tf_exit( ExitERR_misc, "error mapping bpipe: %s",
	     bpipe_strerror( bpipe_errno ) );

  if ( bpipe_write_hdr(bpipe) )
    tf_exit( ExitERR_misc, "error writing header: %s",
	     bpipe_strerror( bpipe_errno ) );

  for( n = 0 ; bpipe_read_dpkts( bpipe, data, 1 ) ; n++ )
  {
    if ( n < c.n )
      bpipe_write_dpkt( bpipe, data, bpo );
    else if ( Abort_IfGreaterThan == c.abort )
    {
      tf_exit( ExitERR_misc, "number of input rays greater than requested number\n" );
    }
  }

  if ( bpipe_errno != BPNOERROR )
    tf_exit( ExitERR_misc, "error whilst reading photon %d: %s", n,
	     bpipe_strerror( bpipe_errno ) );

  if ( n < c.n && Abort_IfLessThan == c.abort )
      tf_exit( ExitERR_misc, "number of input rays less than requested number\n" );


  bpipe_delete( bpipe );

  tf_leave();
}
